
       subroutine modrthomas(n,a,b,c,d,e)
       implicit none


!  subroutine to use Thomas's algorithm to solve tridiagonal matrix
!   Dale U. von Rosenberg, "Methods for the Numerical Solution of
!     Partial Differential Equations, Am. Elsevier Pub., 1969, pg. 113
!      a(i)*u(i-1)+b(i)*u(i)+c(i)*u(i+1)=d(i)
!      a(1)=c(n)==0
!      upon return, d(i)=u(i), a,b, & c modified
! Modified version -- for equation with full last column:
!        (b1   c1                e1   ) x1  = d1
!        (a2   b2   c2           e2   ) x2  = d2
!        (     a3   b3   c3      e3   ) x3  = d3
!         ...................................
!         ...................................
!
!        (             an-1 bn-1 en-1 ) xn-1= dn-1
!        (                  bn   en   ) xn  = dn
!
!
!
       integer :: n,i
       real*8 :: a(n),b(n),c(n),d(n),e(n)

       if (n.lt.1) then
        write(6,*) '***error in thomas -- n = ',n
        return
        else if (n.eq.1) then
         d(1)=d(1)/b(1)
         return
         else
           do i=2,n-1
           b(i)=b(i)-a(i)*c(i-1)/b(i-1)
           enddo
           d(1)=d(1)/b(1)
           e(1)=e(1)/b(1)
           c(1)=c(1)/b(1)
           do i=2,n-1
           d(i)=(d(i)-a(i)*d(i-1))/b(i)
           e(i)=(e(i)-a(i)*e(i-1))/b(i)
           c(i)=c(i)/b(i)
           enddo
           e(n)=e(n)-b(n)*e(n-1)
           d(n)=d(n)-b(n)*d(n-1)
           d(n)=d(n)/e(n)
           d(n-1)=d(n-1)-e(n-1)*d(n)
           do i=n-2,1,-1
           d(i)=d(i)-c(i)*d(i+1)-e(i)*d(n)
           enddo
         endif
         return
          end
