      subroutine sepenergyanal(nz,n,h,mxo,occ,norbit,qcore,qvale,gauss,vlc, &
       vlocfac,hatpot,nlst,llst,nmx,r,twfn,itype,coreden,vden1,tvden1,tvden,dum, &
       cden,nbase,nos,nop,nod,nof,nog,otp,otphi,ophi,mxb,mxnl,tij,qij, &
       am,bm,tv,vcoul,texc,evale,mxwk,wk,iiirc)
!  program to calculate the core and valence contributions to the
!   total energy -- only valence contributions for separable (paw)
!   form of hamiltonian
!    nz = nuclear charge
!    n = number of mesh points
!    h = step size between mesh points ( in bohr)
!    mxo = maximum number of orbitals
!    norbit = actual number of valence orbital
!    llst(mxo) = l for each valence orbital
!    nlst(mxo) = principal quantum number for each valence orbital
!    occ(mxo) = occupation of each valence orbital
!    twfn =  smooth wavefunction (input)
!    itype = exchange-correlation type index
!    coreden = core density -- input
!    tvden = smooth valence density  -- calculated
!    vden1 = one center all-electron valence density  -- calculated
!    tvden1 = one center smooth valence density  -- calculated
!    cden  = gaussian density corresponding to parameter gauss -- n^
!    tv= valence kinetic energy
!    vcoul = valence coulomb energy
!    texc = total exchange-correlation energy - minus pure core
!    evale = tv + vcoul + texc
! reference: U. von Barth and C. D. Gelatt, Phys. Rev. B 21, 2222(1980)
!
      implicit real*8 (a-h,o-z)
      dimension occ(mxo),twfn(nmx,mxo),tvden(nmx),tvden1(nmx)
      dimension cden(nmx),vlc(nmx),hatpot(nmx)
      dimension r(nmx),coreden(nmx),vden1(nmx),dum(nmx),wk(nmx,mxwk)
      dimension otp(nmx,mxo),otphi(nmx,mxo),ophi(nmx,mxo)
      dimension qij(mxb,mxb,mxnl),tij(mxb,mxb,mxnl),am(mxb,mxb),bm(mxb)
      dimension nlst(mxo),llst(mxo)
      pi=4*datan(1.d0)
      if (mxwk.lt.5) then
       write(6,*) 'error in sepenergyanal - mxwk must be gt 5',mxwk
       stop
      endif
!
! kinetic energy
      tv=0
      do io=1,norbit
      l=llst(io)
      istart=0
      call kinetic1(n,h,l,r,twfn(1,io),twfn(1,io),wk,wk(1,2),           &
         wk(1,3),ekin)
      if (l.gt.0) then
      do il=0,l-1
       if (il.eq.0) many=nos
       if (il.eq.1) many=nop
       if (il.eq.2) many=nod
       if (il.eq.3) many=nof
       if (il.eq.4) many=nog
       istart=istart+many
       enddo
       endif
       if (l.eq.0) many=nos
       if (l.eq.1) many=nop
       if (l.eq.2) many=nod
       if (l.eq.3) many=nof
       if (l.eq.4) many=nog
       do ib=1,many
       bm(ib)=overlap(n,h,twfn(1,io),otp(1,istart+ib))
       enddo
       do ib=1,many
       do ic=1,many
         ekin=ekin+bm(ib)*tij(ib,ic,l+1)*bm(ic)
         enddo
         enddo
        tv=tv+occ(io)*ekin
      enddo
      write(6,*) 'valence kinetic energies', tv
!
!   potential energies
!
      call formdensity(n,h,vden1,tvden1,tvden,norbit,llst,mxo,nmx,      &
        twfn,occ,nbase,nos,nop,nod,nof,nog,otp,otphi,ophi,wk,           &
        wk(1,2),qvale,mxnl,mxb,qij,iiirc)
!
      corein=overint(iiirc,h,coreden)
      rcc=r(iiirc)
      coreeff=overint(n,h,coreden)
      write(6,*) 'corein,coreff = ',corein,coreeff
        do i=1,iiirc
        cden(i)=vden1(i)-tvden1(i)
        enddo
        zeff=-nz+coreeff+overint(iiirc,h,cden)
        write(6,*) 'zeff',zeff
        !fac=(4*pi*zeff)/(dsqrt(pi)*gauss)**3
        !do i=1,n
        !cden(i)=fac*(r(i)**2)*ddexp(-(r(i)/gauss)**2)
        !enddo
        ! write(6,*) 'check',overint(n,h,cden)
!
! vden1 + coreden
!
        qqq=overint(iiirc,h,vden1)
        write(6,*) 'int(vden1) = ',qqq
        call poisson(iiirc,h,qqq,vden1,wk(1,2),wk(1,3),wk(1,4),ecoul1)
        qqqc=overint(n,h,coreden)
        write(6,*) ' core check',qqqc,qcore
        call poisson(n,h,qqqc,coreden,wk(1,2),wk(1,3),wk(1,4),xx)
        do i=1,iiirc
         wk(i,2)=(-2*nz+wk(i,2))*vden1(i)/r(i)
         enddo
         ecoul1=ecoul1+overint(iiirc,h,wk(1,2))
!
! tvden + cden
!
        tqv=overint(n,h,tvden)
        write(6,*) 'int(tvden) = ',tqv
        call poisson(n,h,tqv,tvden,wk(1,2),wk(1,3),wk(1,4),tecoul)
          do i=1,n
           dum(i)=(zeff*hatpot(i))/r(i)
           wk(i,2)=dum(i)*tvden(i)
           enddo
           tecoul=tecoul+overint(n,h,wk(1,2))
!
! tvden1 + cden
!
         tqv1=overint(iiirc,h,tvden1)
        write(6,*) 'int(tvden1) = ',tqv1
        call poisson(iiirc,h,tqv1,tvden1,wk(1,2),wk(1,3),wk(1,4),tecoul1)
         do i=1,iiirc
          wk(i,2)=dum(i)*tvden1(i)
           enddo
        tecoul1=tecoul1+overint(iiirc,h,wk(1,2))

!

        write(6,*) 'ecoul1, tecoul, tecoul1',ecoul1,tecoul,tecoul1
        vcoul=ecoul1-tecoul1+tecoul
        write(6,*) ' potentential energy ',vcoul
!
!  exchange-correlation energy
!
         do i=1,iiirc
           dum(i)=tvden(i)
           enddo
         do i=iiirc+1,n
           dum(i)=tvden(i)
           enddo
      call exchcor(n,h,dum,wk,x,vtexc,wk(1,2),wk(1,3),wk(1,4),itype)
         do i=1,iiirc
           dum(i)=tvden1(i)
           enddo
      call exchcor(iiirc,h,dum,wk,x,vtexc1,wk(1,2),wk(1,3),wk(1,4),itype)
      call exchcor(iiirc,h,coreden,wk,x,ccexc,wk(1,2),wk(1,3),wk(1,4),itype)
      do i=1,iiirc
       dum(i)=coreden(i)+vden1(i)
      enddo
      call exchcor(iiirc,h,dum,wk,x,texc,wk(1,2),wk(1,3),wk(1,4),itype)
      write(6,*) 'vtexc,vtexc1,ccexc,texc',vtexc,vtexc1,ccexc,texc

      texc=vtexc-vtexc1+texc-ccexc
      write(6,*) ' exchange-correlation energy', texc
!
!  local potential contribution
!
      vloc=0
      do i=1,iiirc
      dum(i)=vlc(i)*(tvden(i)-tvden1(i))
      enddo
      vloc=overint(iiirc,h,dum)
      write(6,*) 'vlocal contribution ', vloc
!
!  complete result
!
      evale=tv+vcoul+texc+vloc
      write(6,*) 'total valence energy ', evale
      return
       end
